module Pattern where
import Utilities


-------------------------------------------------------
-- Match and substitute
--------------------------------------------------------

-- Replaces a wildcard in a list with the list given as the third argument
substitute :: Eq a => a -> [a] -> [a] -> [a]
substitute _ [] _ = []
substitute wildcard (first:rest) c
	| first == wildcard = c ++ substitute wildcard rest c
	| otherwise = first:substitute wildcard rest c
	

-- Tries to match two lists. If they match, the result consists of the sublist
-- bound to the wildcard in the pattern list.
match :: Eq a => a -> [a] -> [a] -> Maybe [a]
match _ [] [] = Just []
match _ [] _ = Nothing
match _ _ [] = Nothing
match wc (p:ps) (x:xs)
	| wc == p = (singleWildcardMatch (p:ps) (x:xs)) `orElse` (longerWildcardMatch (p:ps) (x:xs))
	| p == x = match wc ps xs
	| otherwise = Nothing


-- Helper function to match
singleWildcardMatch, longerWildcardMatch :: Eq a => [a] -> [a] -> Maybe [a]
-- singleWildcardMatch (wc	:ps) (x:xs) = Nothing
singleWildcardMatch (wc:ps) (x:xs) = mmap (const [x]) (match wc ps xs)

-- longerWildcardMatch (wc:ps) (x:xs) = Nothing
longerWildcardMatch (wc:ps) (x:xs) = mmap (x:) (match wc (wc:ps) xs)



-- Test cases --------------------

testPattern =  "a=*;"
testSubstitutions = "32"
testString = "a=32;"

substituteTest = substitute '*' testPattern testSubstitutions
substituteCheck = substituteTest == testString

matchTest = match '*' testPattern testString
matchCheck = matchTest == Just testSubstitutions



-------------------------------------------------------
-- Applying patterns
--------------------------------------------------------

-- Applying a single pattern
transformationApply :: Eq a => a -> ([a] -> [a]) -> [a] -> ([a], [a]) -> Maybe [a]
transformationApply wildcard function subject (before, after) = mmap   ((substitute wildcard after) . function) (match wildcard before subject) 


-- Applying a list of patterns until one succeeds
transformationsApply :: Eq a => a -> ([a] -> [a]) -> [([a], [a])] -> [a] -> Maybe [a]
-- transformationsApply _ _ [] _ = Nothing
-- transformationsApply wc f ((before, after):rest) subject = (transformationApply wc f subject (before,after)) `orElse` (transformationsApply wc f rest subject)
transformationsApply wc f list subject = foldr1 ( orElse ) (map (transformationApply wc f subject) list)
