module Uppg3 where
import Uppg2 hiding (similarityScore, optAlignments)

similarityScore :: Eq a => [a] -> [a] -> Int
similarityScore xs ys = ssTable!!(length xs)!!(length ys)
	where
		ssTable = [[ ssEntry i j | j<-[0..]] | i<-[0..] ]

		ssEntry :: Int -> Int -> Int
		ssEntry 0 0 = 0
		ssEntry i 0 = scoreSpace + ssTable!!(i-1)!!0
		ssEntry 0 j = scoreSpace + ssTable!!0!!(j-1)
		ssEntry i j =
			maximum [
				twoNonSpace + ssTable!!(i-1)!!(j-1),
				scoreSpace + ssTable!!i!!(j-1),
				scoreSpace + ssTable!!(i-1)!!j
			]
			where
				twoNonSpace
					| xs!!(i-1)==ys!!(j-1) = scoreMatch
					| otherwise = scoreMismatch



attachTails :: a -> a -> [([a],[a])] -> [([a],[a])]
attachTails t1 t2 aList = [(xs++[t1],ys++[t2]) | (xs,ys) <- aList] 



optAlignments :: String -> String -> ([AlignmentType])
optAlignments xs ys = snd $ alignTable!!(length xs)!!(length ys)
	where
		alignTable = [[ alignEntry i j | j<-[0..]] | i<-[0..] ]

		alignEntry :: Int -> Int -> (Int, [AlignmentType])
		alignEntry 0 0 = (0, [("", "")])

		alignEntry i 0 = (scoreSpace + fst prevX, attachTails x '-' $ snd prevX)
			where
				x = xs!!(i-1)
				prevX = alignTable!!(i-1)!!0
				
		alignEntry 0 j = (scoreSpace + fst prevY, attachTails '-' y $ snd prevY)
			where
				y = ys!!(j-1)
				prevY = alignTable!!0!!(j-1)

		alignEntry i j = fixTable $ maximaBy fst 
				[ 
					(twoNonSpace + (fst $ alignTable!!(i-1)!!(j-1)), attachTails x y $ snd $ alignTable!!(i-1)!!(j-1)),
					(scoreSpace + fst prevX, attachTails x '-' $ snd prevX),
					(scoreSpace + fst prevY, attachTails '-' y $ snd prevY)
				]
				
			where
				x = xs!!(i-1)
				y = ys!!(j-1)
				prevX = alignTable!!(i-1)!!j
				prevY = alignTable!!i!!(j-1)
				twoNonSpace
					| xs!!(i-1)==ys!!(j-1) = scoreMatch
					| otherwise = scoreMismatch
				fixTable :: [(Int, [AlignmentType])] -> (Int, [AlignmentType])
				fixTable list = (fst $ list!!0, concat $ map snd list)

